# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/sugar/',
                'doc_host': 'https://velodrome-finance.github.io',
                'git_url': 'https://github.com/velodrome-finance/sugar-sdk/',
                'lib_path': 'sugar'},
  'syms': { 'sugar.abi': { 'sugar.abi.download_contract_abi': ('abi.html#download_contract_abi', 'sugar/abi.py'),
                           'sugar.abi.get_abi': ('abi.html#get_abi', 'sugar/abi.py')},
            'sugar.chains': { 'sugar.chains.AsyncBaseChain': ('chains.html#asyncbasechain', 'sugar/chains.py'),
                              'sugar.chains.AsyncBaseChain.__init__': ('chains.html#asyncbasechain.__init__', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain': ('chains.html#asyncchain', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.__aenter__': ('chains.html#asyncchain.__aenter__', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.__aexit__': ('chains.html#asyncchain.__aexit__', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain._get_prices': ('chains.html#asyncchain._get_prices', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain._get_quotes_for_paths': ( 'chains.html#asyncchain._get_quotes_for_paths',
                                                                                 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.apaginate': ('chains.html#asyncchain.apaginate', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.check_token_allowance': ( 'chains.html#asyncchain.check_token_allowance',
                                                                                 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.deposit': ('chains.html#asyncchain.deposit', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_all_tokens': ('chains.html#asyncchain.get_all_tokens', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_latest_pool_epochs': ( 'chains.html#asyncchain.get_latest_pool_epochs',
                                                                                  'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_pool_by_address': ( 'chains.html#asyncchain.get_pool_by_address',
                                                                               'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_pool_epochs': ('chains.html#asyncchain.get_pool_epochs', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_pools': ('chains.html#asyncchain.get_pools', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_pools_for_swaps': ( 'chains.html#asyncchain.get_pools_for_swaps',
                                                                               'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_prices': ('chains.html#asyncchain.get_prices', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_quote': ('chains.html#asyncchain.get_quote', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.get_raw_pools': ('chains.html#asyncchain.get_raw_pools', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.set_token_allowance': ( 'chains.html#asyncchain.set_token_allowance',
                                                                               'sugar/chains.py'),
                              'sugar.chains.AsyncChain.sign_and_send_tx': ('chains.html#asyncchain.sign_and_send_tx', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.swap': ('chains.html#asyncchain.swap', 'sugar/chains.py'),
                              'sugar.chains.AsyncChain.swap_from_quote': ('chains.html#asyncchain.swap_from_quote', 'sugar/chains.py'),
                              'sugar.chains.AsyncOPChain': ('chains.html#asyncopchain', 'sugar/chains.py'),
                              'sugar.chains.AsyncOPChain.__init__': ('chains.html#asyncopchain.__init__', 'sugar/chains.py'),
                              'sugar.chains.BaseChain': ('chains.html#basechain', 'sugar/chains.py'),
                              'sugar.chains.BaseChain.__init__': ('chains.html#basechain.__init__', 'sugar/chains.py'),
                              'sugar.chains.BaseChainCommon': ('chains.html#basechaincommon', 'sugar/chains.py'),
                              'sugar.chains.Chain': ('chains.html#chain', 'sugar/chains.py'),
                              'sugar.chains.Chain.__enter__': ('chains.html#chain.__enter__', 'sugar/chains.py'),
                              'sugar.chains.Chain.__exit__': ('chains.html#chain.__exit__', 'sugar/chains.py'),
                              'sugar.chains.Chain._get_prices': ('chains.html#chain._get_prices', 'sugar/chains.py'),
                              'sugar.chains.Chain._get_quotes_for_paths': ('chains.html#chain._get_quotes_for_paths', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_all_tokens': ('chains.html#chain.get_all_tokens', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_latest_pool_epochs': ('chains.html#chain.get_latest_pool_epochs', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_pool_by_address': ('chains.html#chain.get_pool_by_address', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_pool_epochs': ('chains.html#chain.get_pool_epochs', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_pools': ('chains.html#chain.get_pools', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_pools_for_swaps': ('chains.html#chain.get_pools_for_swaps', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_prices': ('chains.html#chain.get_prices', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_quote': ('chains.html#chain.get_quote', 'sugar/chains.py'),
                              'sugar.chains.Chain.get_raw_pools': ('chains.html#chain.get_raw_pools', 'sugar/chains.py'),
                              'sugar.chains.Chain.paginate': ('chains.html#chain.paginate', 'sugar/chains.py'),
                              'sugar.chains.Chain.set_token_allowance': ('chains.html#chain.set_token_allowance', 'sugar/chains.py'),
                              'sugar.chains.Chain.sign_and_send_tx': ('chains.html#chain.sign_and_send_tx', 'sugar/chains.py'),
                              'sugar.chains.Chain.swap': ('chains.html#chain.swap', 'sugar/chains.py'),
                              'sugar.chains.Chain.swap_from_quote': ('chains.html#chain.swap_from_quote', 'sugar/chains.py'),
                              'sugar.chains.CommonChain': ('chains.html#commonchain', 'sugar/chains.py'),
                              'sugar.chains.CommonChain.__init__': ('chains.html#commonchain.__init__', 'sugar/chains.py'),
                              'sugar.chains.CommonChain._prepare_prices': ('chains.html#commonchain._prepare_prices', 'sugar/chains.py'),
                              'sugar.chains.CommonChain.account': ('chains.html#commonchain.account', 'sugar/chains.py'),
                              'sugar.chains.CommonChain.filter_pools_for_swap': ( 'chains.html#commonchain.filter_pools_for_swap',
                                                                                  'sugar/chains.py'),
                              'sugar.chains.CommonChain.get_paths_for_quote': ( 'chains.html#commonchain.get_paths_for_quote',
                                                                                'sugar/chains.py'),
                              'sugar.chains.CommonChain.get_pool_paginator': ( 'chains.html#commonchain.get_pool_paginator',
                                                                               'sugar/chains.py'),
                              'sugar.chains.CommonChain.paths_to_pools': ('chains.html#commonchain.paths_to_pools', 'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_pool_epochs': ( 'chains.html#commonchain.prepare_pool_epochs',
                                                                                'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_pools': ('chains.html#commonchain.prepare_pools', 'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_pools_for_swap': ( 'chains.html#commonchain.prepare_pools_for_swap',
                                                                                   'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_price_batcher': ( 'chains.html#commonchain.prepare_price_batcher',
                                                                                  'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_prices': ('chains.html#commonchain.prepare_prices', 'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_quote_batch': ( 'chains.html#commonchain.prepare_quote_batch',
                                                                                'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_quotes': ('chains.html#commonchain.prepare_quotes', 'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_set_token_allowance_contract': ( 'chains.html#commonchain.prepare_set_token_allowance_contract',
                                                                                                 'sugar/chains.py'),
                              'sugar.chains.CommonChain.prepare_tokens': ('chains.html#commonchain.prepare_tokens', 'sugar/chains.py'),
                              'sugar.chains.OPChain': ('chains.html#opchain', 'sugar/chains.py'),
                              'sugar.chains.OPChain.__init__': ('chains.html#opchain.__init__', 'sugar/chains.py'),
                              'sugar.chains.OPChainCommon': ('chains.html#opchaincommon', 'sugar/chains.py'),
                              'sugar.chains.require_async_context': ('chains.html#require_async_context', 'sugar/chains.py'),
                              'sugar.chains.require_context': ('chains.html#require_context', 'sugar/chains.py'),
                              'sugar.chains.safe_format_batched_response': ('chains.html#safe_format_batched_response', 'sugar/chains.py')},
            'sugar.config': { 'sugar.config.ChainSettings': ('config.html#chainsettings', 'sugar/config.py'),
                              'sugar.config.ChainSettings.__repr__': ('config.html#chainsettings.__repr__', 'sugar/config.py'),
                              'sugar.config.ChainSettings.__str__': ('config.html#chainsettings.__str__', 'sugar/config.py'),
                              'sugar.config.get_env': ('config.html#get_env', 'sugar/config.py'),
                              'sugar.config.make_base_chain_settings': ('config.html#make_base_chain_settings', 'sugar/config.py'),
                              'sugar.config.make_op_chain_settings': ('config.html#make_op_chain_settings', 'sugar/config.py'),
                              'sugar.config.make_settings': ('config.html#make_settings', 'sugar/config.py'),
                              'sugar.config.validate_settings': ('config.html#validate_settings', 'sugar/config.py')},
            'sugar.deposit': {'sugar.deposit.Deposit': ('deposit.html#deposit', 'sugar/deposit.py')},
            'sugar.helpers': { 'sugar.helpers.Pair': ('helpers.html#pair', 'sugar/helpers.py'),
                               'sugar.helpers.Timer': ('helpers.html#timer', 'sugar/helpers.py'),
                               'sugar.helpers.Timer.__aenter__': ('helpers.html#timer.__aenter__', 'sugar/helpers.py'),
                               'sugar.helpers.Timer.__aexit__': ('helpers.html#timer.__aexit__', 'sugar/helpers.py'),
                               'sugar.helpers.Timer.__enter__': ('helpers.html#timer.__enter__', 'sugar/helpers.py'),
                               'sugar.helpers.Timer.__exit__': ('helpers.html#timer.__exit__', 'sugar/helpers.py'),
                               'sugar.helpers.Timer.__init__': ('helpers.html#timer.__init__', 'sugar/helpers.py'),
                               'sugar.helpers.amount_to_k_string': ('helpers.html#amount_to_k_string', 'sugar/helpers.py'),
                               'sugar.helpers.amount_to_m_string': ('helpers.html#amount_to_m_string', 'sugar/helpers.py'),
                               'sugar.helpers.apply_slippage': ('helpers.html#apply_slippage', 'sugar/helpers.py'),
                               'sugar.helpers.atime_it': ('helpers.html#atime_it', 'sugar/helpers.py'),
                               'sugar.helpers.chunk': ('helpers.html#chunk', 'sugar/helpers.py'),
                               'sugar.helpers.find_all_paths': ('helpers.html#find_all_paths', 'sugar/helpers.py'),
                               'sugar.helpers.float_to_uint256': ('helpers.html#float_to_uint256', 'sugar/helpers.py'),
                               'sugar.helpers.format_currency': ('helpers.html#format_currency', 'sugar/helpers.py'),
                               'sugar.helpers.format_percentage': ('helpers.html#format_percentage', 'sugar/helpers.py'),
                               'sugar.helpers.get_future_timestamp': ('helpers.html#get_future_timestamp', 'sugar/helpers.py'),
                               'sugar.helpers.normalize_address': ('helpers.html#normalize_address', 'sugar/helpers.py'),
                               'sugar.helpers.time_it': ('helpers.html#time_it', 'sugar/helpers.py')},
            'sugar.pool': { 'sugar.pool.Amount': ('pool.html#amount', 'sugar/pool.py'),
                            'sugar.pool.Amount.amount_in_stable': ('pool.html#amount.amount_in_stable', 'sugar/pool.py'),
                            'sugar.pool.Amount.build': ('pool.html#amount.build', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool': ('pool.html#liquiditypool', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.apr': ('pool.html#liquiditypool.apr', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.from_tuple': ('pool.html#liquiditypool.from_tuple', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.pool_fee_percentage': ( 'pool.html#liquiditypool.pool_fee_percentage',
                                                                              'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.token0_volume': ('pool.html#liquiditypool.token0_volume', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.token1_volume': ('pool.html#liquiditypool.token1_volume', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.total_fees': ('pool.html#liquiditypool.total_fees', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.tvl': ('pool.html#liquiditypool.tvl', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.volume': ('pool.html#liquiditypool.volume', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPool.volume_pct': ('pool.html#liquiditypool.volume_pct', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPoolEpoch': ('pool.html#liquiditypoolepoch', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPoolEpoch.epoch_date': ('pool.html#liquiditypoolepoch.epoch_date', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPoolEpoch.from_tuple': ('pool.html#liquiditypoolepoch.from_tuple', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPoolEpoch.total_fees': ('pool.html#liquiditypoolepoch.total_fees', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPoolEpoch.total_incentives': ( 'pool.html#liquiditypoolepoch.total_incentives',
                                                                                'sugar/pool.py'),
                            'sugar.pool.LiquidityPoolForSwap': ('pool.html#liquiditypoolforswap', 'sugar/pool.py'),
                            'sugar.pool.LiquidityPoolForSwap.from_tuple': ('pool.html#liquiditypoolforswap.from_tuple', 'sugar/pool.py'),
                            'sugar.pool.Price': ('pool.html#price', 'sugar/pool.py'),
                            'sugar.pool.Price.pretty_price': ('pool.html#price.pretty_price', 'sugar/pool.py'),
                            'sugar.pool.symbol': ('pool.html#symbol', 'sugar/pool.py')},
            'sugar.price': { 'sugar.price.Price': ('price.html#price', 'sugar/price.py'),
                             'sugar.price.Price.pretty_price': ('price.html#price.pretty_price', 'sugar/price.py')},
            'sugar.quote': { 'sugar.quote.PreparedRoute': ('quote.html#preparedroute', 'sugar/quote.py'),
                             'sugar.quote.PreparedRoute.encoded': ('quote.html#preparedroute.encoded', 'sugar/quote.py'),
                             'sugar.quote.Quote': ('quote.html#quote', 'sugar/quote.py'),
                             'sugar.quote.Quote.amount_in': ('quote.html#quote.amount_in', 'sugar/quote.py'),
                             'sugar.quote.Quote.from_token': ('quote.html#quote.from_token', 'sugar/quote.py'),
                             'sugar.quote.Quote.path': ('quote.html#quote.path', 'sugar/quote.py'),
                             'sugar.quote.Quote.to_token': ('quote.html#quote.to_token', 'sugar/quote.py'),
                             'sugar.quote.QuoteInput': ('quote.html#quoteinput', 'sugar/quote.py'),
                             'sugar.quote.QuoteInput.from_tuple': ('quote.html#quoteinput.from_tuple', 'sugar/quote.py'),
                             'sugar.quote.QuoteInput.route': ('quote.html#quoteinput.route', 'sugar/quote.py'),
                             'sugar.quote.QuoteInput.to_tuple': ('quote.html#quoteinput.to_tuple', 'sugar/quote.py'),
                             'sugar.quote.pack_path': ('quote.html#pack_path', 'sugar/quote.py')},
            'sugar.swap': { 'sugar.swap.CommandType': ('swap.html#commandtype', 'sugar/swap.py'),
                            'sugar.swap.RoutePlanner': ('swap.html#routeplanner', 'sugar/swap.py'),
                            'sugar.swap.RoutePlanner.__init__': ('swap.html#routeplanner.__init__', 'sugar/swap.py'),
                            'sugar.swap.RoutePlanner.add_command': ('swap.html#routeplanner.add_command', 'sugar/swap.py'),
                            'sugar.swap.RoutePlanner.get_encoded_commands': ( 'swap.html#routeplanner.get_encoded_commands',
                                                                              'sugar/swap.py'),
                            'sugar.swap.RoutePlanner.get_encoded_inputs': ('swap.html#routeplanner.get_encoded_inputs', 'sugar/swap.py'),
                            'sugar.swap.RoutePlanner.get_pretty_encoded_inputs': ( 'swap.html#routeplanner.get_pretty_encoded_inputs',
                                                                                   'sugar/swap.py'),
                            'sugar.swap.setup_planner': ('swap.html#setup_planner', 'sugar/swap.py')},
            'sugar.token': { 'sugar.token.Token': ('token.html#token', 'sugar/token.py'),
                             'sugar.token.Token.from_tuple': ('token.html#token.from_tuple', 'sugar/token.py'),
                             'sugar.token.Token.is_native': ('token.html#token.is_native', 'sugar/token.py'),
                             'sugar.token.Token.make_native_token': ('token.html#token.make_native_token', 'sugar/token.py'),
                             'sugar.token.Token.value_from_bigint': ('token.html#token.value_from_bigint', 'sugar/token.py')}}}
