# AUTOGENERATED! DO NOT EDIT! File to edit: ../src/deposit.ipynb.

# %% auto 0
__all__ = ['Deposit']

# %% ../src/deposit.ipynb 2
from dataclasses import dataclass
from .pool import LiquidityPool

# %% ../src/deposit.ipynb 3
@dataclass(frozen=True)
class Deposit:
    """Data class for Deposits"""
    # pool to deposit into
    pool: LiquidityPool
    # amount of token0 for deposit
    amount_token0: float
